/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 reception library
* Version		: v1.2
* Last updated	: 21.04.07
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version2 of
 the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 General Public License for more details.

 If you have no copy of the GNU General Public License, write to the
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 For other license models, please contact the author.

;***************************************************************************/

#include <avr/io.h>
#include "lib_dmx_in.h"

// ********************* local definitions *********************

enum {IDLE, BREAK, STARTB, STARTADR};			//DMX states

		 uint8_t 	 gDmxState;
		 uint8_t 	*gDmxPnt;
		 uint16_t	 DmxCount;


// *************** DMX Reception Initialisation ****************
void init_DMX(void)
{
#ifdef USE_DIP
DDRC= 0;										//set up DIPs
PORTC= 0xFF;
DDRE &= ~((1<<2)|(1<<1));
PORTE |= (1<<2)|(1<<1);
#endif

DDRD |= (1<<2);
PORTD &= ~(1<<2);								//enable reception
//UBRRH  = 0;
UBRR0H = 0;	// NEW
//UBRRL  = ((F_OSC/4000)-1);						//250kbaud, 8N2
UBRR0L  = ((F_OSC/4000)-1); // NEW						//250kbaud, 8N2
//UCSRC |= (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSR0C |= (3<<UCSZ01)|(1<<USBS0); // NEW
//UCSRB |= (1<<RXEN)|(1<<RXCIE);
UCSR0B |= (1<<RXEN0)|(1<<RXCIE0);	// NEW
gDmxState= IDLE;

uint8_t i;
for (i=0; i<DMX_CHANNELS; i++)
	{
	DmxField[i]= 0;
	}

}


// ************* get DMX start address **************
void get_dips(void)
{
#ifdef USE_DIP
uint16_t Temp= (255-PINC);						//invert DIP state
if (!(PINE &(1<<2)))
	{
	Temp= Temp +256;							//9th bit
	}
if (Temp != 0)
	{
	DmxAddress= Temp;
	if (!(UCSRB &(1<<RXCIE)))					//if receiver was disabled -> enable and wait for break
		{
		gDmxState= IDLE;
		UCSRB |= (1<<RXCIE);
		}
	}
else
	{
	UCSRB &= ~(1<<RXCIE);						//disable Receiver if start address = 0
	for (Temp=0; Temp<DMX_CHANNELS; Temp++)
		{
		DmxField[Temp]= 0;
		}
	}
#endif
	DmxAddress = 1;
}


// *************** DMX Reception ISR ****************
ISR (USART0_RX_vect)
{
//uint8_t USARTstate= UCSRA;						//get state
uint8_t USARTstate= UCSR0A;  //NEW						//get state
//uint8_t DmxByte= UDR;							//get data
uint8_t DmxByte= UDR0;		// NEW					//get data
uint8_t DmxState= gDmxState;					//just get once from SRAM!!!

//if (USARTstate &(1<<FE))						//check for break
if (USARTstate &(1<<FE0)) // NEW						//check for break
	{
	//UCSRA &= ~(1<<FE);							//reset flag
	UCSR0A &= ~(1<<FE0);	//NEW						//reset flag
	DmxCount= DmxAddress;						//reset frame counter
	gDmxState= BREAK;
	}

else if (DmxState == BREAK)
	{
	if (DmxByte == 0)
		{
		gDmxState= STARTB;						//normal start code detected
		gDmxPnt= ((uint8_t*)DmxField +1);
		}
	else gDmxState= IDLE;
	}

else if (DmxState == STARTB)
	{
	if (--DmxCount == 0)						//start address reached?
		{
		gDmxState= STARTADR;
		DmxField[0]= DmxByte;
		}
	}

else if (DmxState == STARTADR)
	{
	uint8_t *DmxPnt;
	DmxPnt= gDmxPnt;
	*DmxPnt= DmxByte;
	if (++DmxPnt >= (DmxField +DMX_CHANNELS)) 	//all ch received?
		{
		gDmxState= IDLE;
		}
	else gDmxPnt= DmxPnt;
	}
}


